/* =======================================
	Temperature Distribution
	Thermal Diffusion Equation
	--------------------------
	Author: Dai Weifeng
	Date: 2013.09.26
	--------------------------
	Geometry File:	stick.geo
	Mesh File:	stick.msh
   ======================================= */

/* ==== Group Definition ==== */

Group{
	// Example: 
	// heatsource = Region[1001]
	// coldsource = Region[1002]
	heatsource = Region[1001];
	coldsource = Region[1002];
	sidesurface = Region[1003];
	tot_volume = Region[10001];
	tot_surface = Region[{heatsource, coldsource, sidesurface}];
	tot = Region[{tot_surface, tot_volume}];
}
/* ==== End of Group Definition ==== */

/* ==== Function Definition ==== */
Function{
	// Constants
	sigma = 5.67e-8;	// Stephan - Boltzmann Constant
	room_temp = 300;	// Room Temp. (Background/Environment Temp.)
	Tref = room_temp;	// Reference Temp. for Emission and Convection
	Tinit = room_temp;	// Initialized Temp. for Total part
	CHTC_AIR = 15.0;	// Convection Heat Transfer Coefficient
	// Time-domain Iteration Parameter
	t0 = 0 ;			// starting time
	t1 = 100 ;			// ending time
	dt = 1.0 ;			// time interval of one step
	iter_theta = 1.0 ;	// iter_theta = 1.0 means Euler Iteration ; iter_theta = 0.5 means Crank-Nicholson Iteration (in the Time Domain)
	
	//
	TCD[tot] = 1.0;
	CV[tot] = 1.0;
	D[] = 1.0;
	
	// chip size
	Stick_Width = 0.1;
	Stick_Length = 0.1;
	// Stick_Height = 1.0;
	
	J = 1.0;
	Emis[coldsource] = 1.0;
	js[heatsource] = J/(Stick_Width*Stick_Length);
	
}
/* ==== End of Function Definition ==== */

/* ==== Constraint Definition ==== */
Constraint {
	{ Name T_INIT ;
		Case {
			{ Region tot_volume ; Type Init ; Value Tinit ; }
			{ Region tot_surface ; Type Init ; Value Tinit ; }
		}
	}
	
	{ Name T_FIX ;
		Case {
			{ Region coldsource ; Value room_temp; }
			{ Region heatsource ; Value room_temp + 100; }
		}
	}
}
/* ==== End of Constraint Definition ==== */


/* ==== Jacobian ==== */
// General Example
Jacobian {
	{ Name Jac ;
		Case {
			{ Region tot_volume ; Jacobian Vol ; }
			{ Region tot_surface ; Jacobian Sur ; }
		}
	}
}
/* ==== End of Jacobian ==== */


/* ==== Integration ==== */
// Example

Integration {
	{ Name I1;
		Case { 
			{ Type Gauss ;
				Case {
					{GeoElement Triangle ; NumberOfPoints 3 ;}
					{GeoElement Quadrangle ; NumberOfPoints 4;}
					{GeoElement Tetrahedron ; NumberOfPoints 4 ;}
					{GeoElement Hexahedron  ; NumberOfPoints 8 ;}
					{GeoElement Prism ; NumberOfPoints 9 ;}
				}
			}
		}
	}
}
/* ==== End of Integration ==== */

/* ==== Function Space ==== */
FunctionSpace {
	{ Name DGrad_T ; Type Form0 ;
		BasisFunction {
			{ Name sn ; NameOfCoef Tn ; Function BF_Node ;
			  Support tot ; Entity NodesOf[ tot ] ; }			// tot (means all domains here) should be defined in "Groups".
		}
		Constraint {
			{ NameOfCoef Tn ; EntityType NodesOf ; NameOfConstraint T_INIT ;}	// 
		}
	}
	
	{ Name Sta_T ; Type Form0 ;
		BasisFunction {
			{ Name sn ; NameOfCoef Tn ; Function BF_Node ;
			  Support tot ; Entity NodesOf[ tot ] ; }
		}
		Constraint {
			{ NameOfCoef Tn ; EntityType NodesOf ; NameOfConstraint T_FIX ; }
		}
	}
}
/* ==== End of Function Space */

/* ==== Formulation ==== */
Formulation {
	// Here is an example for transient thermal problems
	{ Name Dyn_T ; Type FemEquation ;
		Quantity {
			{ Name T ; Type Local ; NameOfSpace DGrad_T ; }
		}
		Equation {
			Galerkin { Dt[ CV[] * Dof{T}, {T} ] ;				// CV[] = Volumetric heat capacity
				In tot ; Jacobian Jac ; Integration I1 ;	// CV[] = Density[] * Cm[]
			}
			
			Galerkin { [ TCD[] * Dof{Grad T}, {Grad T} ] ;			// TCD[] = Thermal conductivity
				In tot ; Jacobian Jac ; Integration I1 ;	// TCD[] = CV[] * D[], D[] here is diffusivity
			}
			Galerkin { [ -js[], {T} ] ;					// Joule heat source density
				In heatsource  ; Jacobian Jac ; Integration I1 ;	// 
			}
			
			Galerkin { [ sigma * Emis[] * ({T}^4 - Tref^4), {T} ] ;		// sigma = Stephan - Boltzman Constant for the thermal radiation equation
				In coldsource ; Jacobian Jac ; Integration I1 ;	//
			}
			
			Galerkin { [ CHTC_AIR * ({T} - Tref), {T} ] ;			// Thermal convection
				In coldsource ; Jacobian Jac ; Integration I1 ;
			}
		}
	}
	
	{ Name Sta_T ; Type FemEquation ;
		Quantity {
			{ Name T ; Type Local ; NameOfSpace Sta_T ; }
		}
		Equation {
			Galerkin { [ -D[] * Dof{Grad T}, {Grad T} ] ;
			In tot ; Jacobian Jac ; Integration I1 ;
			}
		}
	}
}
/* ==== End of Formulation ==== */

/* ==== Resolution ==== */
Resolution {
	{ Name Dyn_T ;
		System {
			{ Name T ; NameOfFormulation Dyn_T ; }
		}
		Operation {
			InitSolution[T];
			SaveSolution[T];
			GenerateSeparate[T];
			TimeLoopTheta[t0, t1, dt, iter_theta]{
				Generate[T];
				Solve[T];
				SaveSolution[T];
			}
			
		}
	}
	
	{ Name Sta_T ;
		System {
			{ Name T ; NameOfFormulation Sta_T ;}
		}
		Operation {
			Generate[T];
			Solve[T];
			SaveSolution[T];
		}
	}
	
}
/* ==== End of Resolution ==== */

/* ==== Post Processing ==== */
PostProcessing {
	
	{ Name Dyn_T ; NameOfFormulation Dyn_T ;
		Quantity {
			{ Name T ; Value {Local {[{T}] ; In tot ; Jacobian Jac ; } } }	// Total domain should be defined in "Group"
		}
	}
	
	{ Name Sta_T ; NameOfFormulation Sta_T ;
		Quantity {
			{ Name T ; Value {Local {[{T}] ; In tot ; Jacobian Jac ; } } }
		}
	}
}
/* ==== End of Post Processing ==== */

/* ==== Post Operation ==== */

PostOperation {
	
	{ Name T ; NameOfPostProcessing Dyn_T ;
		Operation {
			Print[ T, OnElementsOf tot_surface, File "T_tot_surface.pos" ] ;
			Print[ T, OnElementsOf tot_volume, File "T_tot_volume.pos" ] ;
			/*
			Print[ T, OnSection{ {0,0,0} {0.05,0,0} {0,0,0.5} }, File "T_cross_section_XZ.pos" ] ;
			Print[ T, OnSection{ {0,0,0} {0,0.05,0} {0,0,0.5} }, File "T_cross_section_YZ.pos" ] ;
			Print[ T, OnSection{ {0,0,0} {0.05,0,0} {0,0.05,0} }, File "T_cross_section_XY.pos" ] ;
			*/
		}
	}
}

/* ==== End of Post Operation ==== */